@@ -1,10 +1,10 @@
 package Net::DNS::Packet;
 
 #
-# $Id: Packet.pm 1360 2015-06-15 09:58:53Z willem $
+# $Id: Packet.pm 1365 2015-06-26 08:46:01Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1360 $)[1];
+$VERSION = (qw$LastChangedRevision: 1365 $)[1];
 
 
 =head1 NAME
@@ -648,15 +648,15 @@ on the final packet in the absence of more specific information.
 sub verify {
 	my $self = shift;
 
-	my $sig = $self->sigrr || return undef;
-	return $sig->verify( $self, @_ );
+	my $sig = $self->sigrr;
+	return $sig ? $sig->verify( $self, @_ ) : undef;
 }
 
 sub verifyerr {
 	my $self = shift;
 
-	my $sig = $self->sigrr || return 'not signed';
-	return $sig->vrfyerrstr;
+	my $sig = $self->sigrr;
+	return $sig ? $sig->vrfyerrstr : 'not signed';
 }
 
 
