@@ -1,10 +1,10 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1362 2015-06-23 08:47:14Z willem $
+# $Id: OPT.pm 1365 2015-06-26 08:46:01Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1362 $)[1];
+$VERSION = (qw$LastChangedRevision: 1365 $)[1];
 
 
 use strict;
@@ -165,7 +165,8 @@ sub option {
 
 sub defined {
 	my $self = shift;
-	scalar grep CORE::defined( $self->{$_} ), qw(size flags rcode option);
+	my @defd = grep $self->{$_}, qw(size flags rcode option);
+	return scalar @defd;
 }
 
 
