@@ -1,10 +1,10 @@
 package Net::DNS::RR::SSHFP;
 
 #
-# $Id: SSHFP.pm 1337 2015-03-26 18:53:04Z willem $
+# $Id: SSHFP.pm 1362 2015-06-23 08:47:14Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1337 $)[1];
+$VERSION = (qw$LastChangedRevision: 1362 $)[1];
 
 
 use strict;
@@ -19,7 +19,7 @@ Net::DNS::RR::SSHFP - DNS SSHFP resource record
 
 use integer;
 
-use constant BABBLE => eval { require Digest::BubbleBabble; } || 0;
+use constant BABBLE => ref( eval { require Digest::BubbleBabble; \1; } );
 
 
 sub decode_rdata {			## decode rdata from wire-format octet string
@@ -62,7 +62,7 @@ sub algorithm {
 	my $self = shift;
 
 	$self->{algorithm} = 0 + shift if scalar @_;
-	return $self->{algorithm} || 0;
+	$self->{algorithm} || 0;
 }
 
 
@@ -70,7 +70,7 @@ sub fptype {
 	my $self = shift;
 
 	$self->{fptype} = 0 + shift if scalar @_;
-	return $self->{fptype} || 0;
+	$self->{fptype} || 0;
 }
 
 
