@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1204 2014-05-20 20:52:47Z willem $
+# $Id: Base.pm 1345 2015-05-19 10:07:43Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1204 $)[1];
+$VERSION = (qw$LastChangedRevision: 1345 $)[1];
 
 
 use strict;
@@ -15,18 +15,21 @@ use Socket;
 use IO::Socket;
 use IO::Select;
 
-require Net::DNS::Packet;
-require Net::DNS::RR;
+use Net::DNS::RR;
+use Net::DNS::Packet;
 
-use constant DNSSEC => eval { require Net::DNS::RR::DS; } || 0;
 use constant INT16SZ  => 2;
 use constant PACKETSZ => 512;
 
+use constant UTIL => eval { require Scalar::Util; } || 0;
+sub tainted { return UTIL ? Scalar::Util::tainted(shift) : undef }
+sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
+
 
 #
 #  A few implementation notes wrt IPv6 support.
 #
-#  In general we try to be gracious to those stacks that do not have ipv6 support.
+#  In general we try to be gracious to those stacks that do not have IPv6 support.
 #  We test that by means of the availability of Socket6 and IO::Socket::INET6
 #
 
@@ -43,8 +46,8 @@ use constant PACKETSZ => 512;
 #  call to translate IP addresses to socketaddress
 
 
-#  Two configuration flags, force_v4 and prefer_v6, are provided to
-#  control IPv6 behaviour for test purposes.
+#  Three configuration flags, force_v4, prefer_v6 and force_v6,
+#  are provided to control IPv6 behaviour for test purposes.
 
 
 # Olaf Kolkman, RIPE NCC, December 2003.
@@ -85,17 +88,16 @@ BEGIN {
 		errorstring	=> 'unknown error or no error',
 		tsig_rr		=> undef,
 		answerfrom	=> '',
-		querytime	=> undef,
 		tcp_timeout	=> 120,
 		udp_timeout	=> undef,
-		axfr_sel	=> undef,
 		persistent_tcp	=> 0,
 		persistent_udp	=> 0,
 		dnssec		=> 0,
+		adflag		=> 0,	# see RFC6840, 5.7
+		cdflag		=> 0,	# see RFC6840, 5.9
 		udppacketsize	=> 0,	# value bounded below by PACKETSZ
-		cdflag		=> 0,	# this is only used when {dnssec} == 1
-		adflag		=> 1,	# this is only used when {dnssec} == 1
-		force_v4	=> 0,	# only relevant when we have v6 support
+		force_v4	=> 0,	# only relevant if IPv6 is supported
+		force_v6	=> 0,	#
 		prefer_v6	=> 0,	# prefer v6, otherwise prefer v4
 		ignqrid		=> 0,	# normally packets with non-matching ID
 					# or with the qr bit on are thrown away,
@@ -106,7 +108,7 @@ BEGIN {
 					# This may be a temporary feature
 		);
 
-	# If we're running under a SOCKSified Perl, use TCP instead of UDP
+	# If running under a SOCKSified Perl, use TCP instead of UDP
 	# and keep the sockets open.
 	if ( $Config::Config{'usesocks'} ) {
 		$defaults{'usevc'}	    = 1;
@@ -118,9 +120,9 @@ BEGIN {
 	sub defaults { return $defaults; }
 }
 
-# These are the attributes that we let the user specify in the new().
-# We also deprecate access to these with AUTOLOAD (some may be useful).
-my %public_attr = map { $_ => 1 } qw(
+# These are the attributes that the user may specify in the new() constructor.
+my %public_attr = map { $_ => $_ } qw(
+		nameserver
 		nameservers
 		port
 		srcaddr
@@ -141,6 +143,9 @@ my %public_attr = map { $_ => 1 } qw(
 		persistent_tcp
 		persistent_udp
 		dnssec
+		adflag
+		cdflag
+		prefer_v6
 		ignqrid
 		);
 
@@ -149,43 +154,59 @@ my $initial;
 
 sub new {
 	my $class = shift;
-	my %args = @_ unless scalar(@_) % 2;
+	my %args = ( scalar(@_) % 2 ) ? () : @_;
 
 	my $self;
+	my $base = $class->defaults;
+	my $init = $initial;
+	$initial ||= bless {%$base}, $class;
 	if ( my $file = $args{'config_file'} ) {
-		my $base = $initial || $class->defaults;
-		$self = bless {%$base}, $class;
+		$self = bless {%$initial}, $class;
 		$self->read_config_file($file);			# user specified config
+		$self->nameservers( _untaint $self->nameservers );
+		my @searchlist = _untaint $base->searchlist;
+		$base->searchlist( _untaint $base->domain || () ) unless @searchlist;
+		$base->domain(@searchlist) unless $base->domain;
+		%$base = %$self unless $init;			# define default configuration
+
+	} elsif ($init) {
+		$self = bless {%$base}, $class;
 
 	} else {
-		my $base = $class->defaults;
-		$initial ||= {%$base} && do {
-			$class->init();				# system-wide config
-		};
+		$class->init();					# define default configuration
+		my @searchlist = $base->searchlist;
+		$base->searchlist( $base->domain || () ) unless @searchlist;
+		$base->domain(@searchlist) unless $base->domain;
 		$self = bless {%$base}, $class;
 	}
 
-
 	while ( my ( $attr, $value ) = each %args ) {
 		next unless $public_attr{$attr};
-
-		if ( $attr eq 'nameservers' || $attr eq 'searchlist' ) {
-
-			croak "usage: Net::DNS::Resolver->new( $attr => [ ... ] )"
-					unless UNIVERSAL::isa( $value, 'ARRAY' );
-		}
-
-		if ( $attr eq 'nameservers' ) {
-			$self->nameservers(@$value);
-		} else {
-			$self->{$attr} = $value;
-		}
+		my $ref = ref($value);
+		croak "usage: Net::DNS::Resolver->new( $attr => [ ... ] )"
+				if $ref ne 'ARRAY' && $attr =~ /servers|list/;
+		$self->$attr( $ref ? @$value : $value );
 	}
 
 	return $self;
 }
 
 
+#
+# $class->read_env() or $object->read_env()
+#
+my %resolv_conf = (			## map traditional resolv.conf option names
+	attempts => 'retry',
+	inet6	 => 'prefer_v6',
+	timeout	 => 'retrans',
+	);
+
+my %env_option = (			## any resolver attribute except as listed below
+	%public_attr,
+	%resolv_conf,
+	map { $_ => 0 } qw(nameserver nameservers domain searchlist ignqrid),
+	);
+
 sub read_env {
 	my ($invocant) = @_;
 	my $config = ref($invocant) ? $invocant : $invocant->defaults;
@@ -199,15 +220,16 @@ sub read_env {
 	if ( exists $ENV{RES_OPTIONS} ) {
 		foreach ( map split, $ENV{RES_OPTIONS} ) {
 			my ( $name, $val ) = split( m/:/, $_, 2 );
+			my $attribute = $env_option{$name} || next;
 			$val = 1 unless defined $val;
-			$config->{$name} = $val if exists $config->{$name};
+			$config->$attribute($val);
 		}
 	}
 }
 
 
 #
-# $class->read_config_file($filename) or $object->read_config_file($file)
+# $class->read_config_file($filename) or $object->read_config_file($filename)
 #
 sub read_config_file {
 	my ( $invocant, $file ) = @_;
@@ -218,33 +240,41 @@ sub read_config_file {
 	local *FILE;
 
 	open( FILE, $file ) or croak "Could not open $file: $!";
-	local $/ = "\n";
 
+	local $SIG{__WARN__} = sub { die @_ };
+	local $_;
 	while (<FILE>) {
-		s/\s*[;#].*$//;					# strip comment
-		next unless m/\S/;				# skip empty line
-		s/^\s+//;					# strip leading space
+		s/[;#].*$//;					# strip comments
 
 		/^nameserver/ && do {
-			my ( $keyword, @ip ) = split;
-			push @ns, map { $_ eq '0' ? '0.0.0.0' : $_ } @ip;
+			my ( $keyword, @ip ) = grep defined, split;
+			push @ns, map $_ eq '0' ? '0.0.0.0' : $_, @ip;
+			next;
+		};
+
+		/^option/ && do {
+			my ( $keyword, $option ) = grep defined, split;
+			my ( $name, $val ) = split( m/:/, $option, 2 );
+			my $attribute = $resolv_conf{$name} || next;
+			$val = 1 unless defined $val;
+			$config->$attribute($val);
 			next;
 		};
 
 		/^domain/ && do {
-			my $keyword;
-			( $keyword, $config->{domain} ) = split;
+			my ( $keyword, $domain ) = grep defined, split;
+			$config->domain($domain);
 			next;
 		};
 
 		/^search/ && do {
-			my ( $keyword, @searchlist ) = split;
-			$config->{searchlist} = \@searchlist;
+			my ( $keyword, @searchlist ) = grep defined, split;
+			$config->searchlist(@searchlist);
 			next;
 		};
 	}
 
-	close FILE || croak "Could not close $file: $!";
+	close(FILE) || croak "close $file: $!";
 
 	$config->nameservers(@ns);
 }
@@ -256,10 +286,11 @@ sub print { print shift->string; }
 sub string {
 	my $self = shift;
 
-	my $timeout   = $self->{'tcp_timeout'} ? $self->{'tcp_timeout'}		   : 'indefinite';
-	my $INET6line = $has_inet6	       ? "prefer_v6 = $self->{prefer_v6}"  : '(no IPv6 transport)';
-	my $ignqrid   = $self->{'ignqrid'}     ? 'ACCEPTING ALL PACKETS (IGNQRID)' : '';
-	my @nslist    = $self->nameservers();
+	my $timeout = $self->{'tcp_timeout'} || 'indefinite';
+	my $IP6line = "prefer_v6\t= $self->{prefer_v6}\tforce_v6 = $self->{force_v6}";
+	my $IP6conf = $has_inet6 ? $IP6line : '(no IPv6 transport)';
+	my @nslist  = $self->nameservers();
+	my $ignqrid = $self->{'ignqrid'} ? 'ACCEPTING ALL PACKETS (IGNQRID)' : '';
 	return <<END;
 ;; RESOLVER state:
 ;;  domain	= $self->{domain}
@@ -269,12 +300,12 @@ sub string {
 ;;  srcport	= $self->{srcport}
 ;;  srcaddr	= $self->{srcaddr}
 ;;  tcp_timeout = $timeout
-;;  retrans	= $self->{retrans}	retry     = $self->{retry}
-;;  usevc	= $self->{usevc}	stayopen  = $self->{stayopen}
-;;  defnames	= $self->{defnames}	dnsrch    = $self->{dnsrch}
-;;  recurse	= $self->{recurse}	igntc     = $self->{igntc}
-;;  force_v4	= $self->{force_v4}	$INET6line
-;;  debug	= $self->{debug}	$ignqrid
+;;  retrans	= $self->{retrans}	retry    = $self->{retry}
+;;  usevc	= $self->{usevc}	stayopen = $self->{stayopen}
+;;  defnames	= $self->{defnames}	dnsrch   = $self->{dnsrch}
+;;  recurse	= $self->{recurse}	igntc    = $self->{igntc}
+;;  debug	= $self->{debug}	force_v4 = $self->{force_v4}	$ignqrid
+;;  $IP6conf
 END
 
 }
@@ -294,40 +325,36 @@ sub empty_searchlist {
 
 sub nameservers {
 	my $self = shift;
+	$self = $self->defaults unless ref($self);
 
 	my ( @ipv4, @ipv6 );
 	foreach my $ns (@_) {
-		next unless length($ns);
-		push( @ipv6, $ns ) && next if _ip_is_ipv6($ns);
-		push( @ipv4, $ns ) && next if _ip_is_ipv4($ns);
+		croak 'nameservers: invalid argument' unless $ns;
+		do { push @ipv6, $ns; next } if _ip_is_ipv6($ns);
+		do { push @ipv4, $ns; next } if _ip_is_ipv4($ns);
 
-		my $defres = Net::DNS::Resolver->new(
+		my $defres = ref($self)->new(
 			udp_timeout => $self->udp_timeout,
-			tcp_timeout => $self->tcp_timeout
-			);
-		$defres->{debug} = $self->{debug};
-
-		my @names;
-		if ( $ns =~ /\./ ) {
-			@names = ($ns);
-		} else {
-			my @suffix = $defres->searchlist;
-			@suffix = grep length, ( $defres->domain ) unless @suffix;
-			@names = map "$ns.$_", @suffix;
-		}
+			tcp_timeout => $self->tcp_timeout,
+			debug	    => $self->{debug} );
+		$defres->{cache} = $self->{cache} if $self->{cache};
 
 		my $packet = $defres->search( $ns, 'A' );
 		$self->errorstring( $defres->errorstring );
-		my @address = cname_addr( [@names], $packet ) if defined $packet;
+		my @names = ($ns);
+		push @names, $packet ? ( map $_->qname, $packet->question ) : ();
+		my @address = $packet ? cname_addr( [@names], $packet ) : ();
 
 		if ($has_inet6) {
 			$packet = $defres->search( $ns, 'AAAA' );
 			$self->errorstring( $defres->errorstring );
-			push @address, cname_addr( [@names], $packet ) if defined $packet;
+			push @names, $packet ? ( map $_->qname, $packet->question ) : ();
+			push @address, $packet ? cname_addr( [@names], $packet ) : ();
 		}
 
-		my %address = map { $_ => 1 } @address;
-		my @unique = keys %address;
+		my %address = map { ( $_ => $_ ) } @address;	# tainted
+		my @unique = values %address;
+		carp "unresolvable name: $ns" unless @unique;
 		push @ipv4, grep _ip_is_ipv4($_), @unique;
 		push @ipv6, grep _ip_is_ipv6($_), @unique;
 	}
@@ -338,20 +365,15 @@ sub nameservers {
 		return unless defined wantarray;
 	}
 
-	my @returnval = @{$self->{nameserver6}} if $has_inet6 && !$self->force_v4();
-	if ( $self->prefer_v6() ) {
-		push @returnval, @{$self->{nameserver4}};
-	} else {
-		unshift @returnval, @{$self->{nameserver4}};
-	}
+	my @ns4 = $self->force_v6 ? () : @{$self->{nameserver4}};
+	my @ns6 = $has_inet6 && !$self->force_v4 ? @{$self->{nameserver6}} : ();
+	my @returnval = $self->prefer_v6 ? ( @ns6, @ns4 ) : ( @ns4, @ns6 );
 
 	return @returnval if scalar @returnval;
 
 	$self->errorstring('no nameservers');
-	if ( scalar( @{$self->{nameserver6}} ) ) {
-		$self->errorstring('IPv6 transport not available') unless $has_inet6;
-		$self->errorstring('unable to use IPv6 transport') if $self->force_v4();
-	}
+	$self->errorstring('IPv4 transport not available') if scalar(@ns4) < scalar @{$self->{nameserver4}};
+	$self->errorstring('IPv6 transport not available') if scalar(@ns6) < scalar @{$self->{nameserver6}};
 	return @returnval;
 }
 
@@ -390,12 +412,24 @@ sub cname_addr {
 # then we use EDNS and $self->{udppacketsize}
 # should be taken as the maximum packet_data length
 sub _packetsz {
-	my $udpsize = shift->{udppacketsize} || PACKETSZ;
+	my $udpsize = shift->{udppacketsize} || 0;
 	return $udpsize > PACKETSZ ? $udpsize : PACKETSZ;
 }
 
+sub answerfrom {
+	my $self = shift;
+	$self->{answerfrom} = shift if scalar @_;
+	return $self->{answerfrom};
+}
+
+sub errorstring {
+	my $self = shift;
+	$self->{errorstring} = shift if scalar @_;
+	return $self->{errorstring};
+}
+
 sub _reset_errorstring {
-	my ($self) = @_;
+	my $self = shift;
 
 	$self->errorstring( $self->defaults->{'errorstring'} );
 }
@@ -405,8 +439,8 @@ sub search {
 	my $self = shift;
 	my $name = shift || '.';
 
-	my $defdomain  = $self->{domain}	  if $self->{defnames};
-	my @searchlist = @{$self->{'searchlist'}} if $self->{dnsrch};
+	my $defdomain  = $self->{defnames} ? $self->{domain}	      : undef;
+	my @searchlist = $self->{dnsrch}   ? @{$self->{'searchlist'}} : ();
 
 	# resolve name by trying as absolute name, then applying searchlist
 	my @list = ( undef, @searchlist );
@@ -442,7 +476,7 @@ sub query {
 	my $name = shift || '.';
 
 	# resolve name containing no dots or colons by appending domain
-	my @suffix = ( $self->{domain} || () ) if $name !~ m/[:.]/ and $self->{defnames};
+	my @suffix = ( $name !~ m/[:.]/ && $self->{defnames} ) ? ( $self->{domain} || () ) : ();
 
 	my $fqname = join '.', $name, @suffix;
 
@@ -497,12 +531,11 @@ sub send_tcp {
 
 
 NAMESERVER: foreach my $ns (@ns) {
-
-		print ";; attempt to send_tcp [$ns]:$dstport  (src port = $srcport)\n"
-				if $self->{'debug'};
 		my $sock;
 		my $sock_key = "$ns:$dstport";
-		my ( $host, $port );
+
+		print ";; send_tcp [$ns]:$dstport  (src port = $srcport)\n" if $self->{'debug'};
+
 		if ( $self->persistent_tcp && $self->{'sockets'}[AF_UNSPEC]{$sock_key} ) {
 			$sock = $self->{'sockets'}[AF_UNSPEC]{$sock_key};
 			print ";; using persistent socket\n"
@@ -615,7 +648,7 @@ sub send_udp {
 
 	my $lastanswer;
 
-	my $stop_time = time + $self->{'udp_timeout'} if $self->{'udp_timeout'};
+	my $stop_time = $self->{'udp_timeout'} ? time + $self->{'udp_timeout'} : undef;
 
 	$self->_reset_errorstring;
 
@@ -648,7 +681,7 @@ sub send_udp {
 
 		my $srcaddr6 = $srcaddr eq '0.0.0.0' ? '::' : $srcaddr;
 
-		print ";; setting up an AF_INET6 UDP socket with srcaddr [$srcaddr6] ... "
+		print ";; setting up AF_INET6 UDP socket with srcaddr [$srcaddr6] ... "
 				if $self->{'debug'};
 
 		# IO::Socket carps on errors if Perl's -w flag is turned on.
@@ -668,10 +701,10 @@ sub send_udp {
 	}
 
 	# Always set up an AF_INET socket.
-	# It will be used if the address family of for the endpoint is V4.
+	# It will be used if the address family of the endpoint is V4.
 
 	unless ( defined( $sock[AF_INET] ) ) {
-		print ";; setting up an AF_INET	 UDP socket with srcaddr [$srcaddr] ... "
+		print ";; setting up AF_INET  UDP socket with srcaddr [$srcaddr] ... "
 				if $self->{'debug'};
 
 		#my $old_wflag = $^W;
@@ -709,8 +742,8 @@ NSADDRESS: foreach my $ns_address ( $self->nameservers() ) {
 
 		# The logic below determines the $dst_sockaddr.
 		# If getaddrinfo is available that is used for both INET4 and INET6
-		# If getaddrinfo is not avialable (Socket6 failed to load) we revert
-		# to the 'classic mechanism
+		# If getaddrinfo is not available (Socket6 failed to load) we revert
+		# to the 'classic' mechanism
 		if ( $has_inet6 && !$self->force_v4() ) {
 
 			# we can use getaddrinfo
@@ -718,22 +751,20 @@ NSADDRESS: foreach my $ns_address ( $self->nameservers() ) {
 								# AI_NUMERICHOST is not available at compile time.
 								# The AI_NUMERICHOST suppresses lookups.
 
-			my $old_wflag = $^W;			#circumvent perl -w warnings about 'udp'
-			$^W = 0;
+			local $^W = 0;				# circumvent perl -w warnings about 'udp'
 
 			my @res = Socket6::getaddrinfo( $ns_address, $dstport, AF_UNSPEC, SOCK_DGRAM, 0,
 				AI_NUMERICHOST );
 
-			$^W = $old_wflag;
-
-			use strict 'subs';
-
-			my ( $sockfamily, $socktype_tmp, $proto_tmp, $dst_sockaddr, $canonname_tmp ) = @res;
-
 			if ( scalar(@res) < 5 ) {
-				die("can't resolve \"$ns_address\" to address");
+				for ($ns_address) {
+					die("invalid scoped address '$_'") if /[%].*$/;
+					die("invalid IP address '$_'");
+				}
 			}
 
+			my ( $sockfamily, $socktype, $proto, $dst_sockaddr, $canonname ) = @res;
+
 			push @ns, [$ns_address, $dst_sockaddr, $sockfamily];
 
 		} else {
@@ -757,16 +788,16 @@ NSADDRESS: foreach my $ns_address ( $self->nameservers() ) {
 	$sel->add( $sock[AF_INET6()] ) if $has_inet6 && defined( $sock[AF_INET6()] ) && !$self->force_v4();
 
 	# Perform each round of retries.
-	for (	my $i = 0 ;
-		$i < $self->{'retry'} ;
-		++$i, $retrans *= 2, $timeout = int( $retrans / ( @ns || 1 ) )
-		) {
+	for ( my $i = 0 ; $i < $self->{'retry'} ; ++$i, $retrans *= 2 ) {
 
-		$timeout = 1 if ( $timeout < 1 );
+		$timeout = int( $retrans / ( scalar @ns || 1 ) );
+		$timeout = 1 if $timeout < 1;
 
 		# Try each nameserver.
 NAMESERVER: foreach my $ns (@ns) {
-			next if defined $ns->[3];
+			my ( $nsname, $nsaddr, $nssockfamily, $err ) = @$ns;
+			next if defined $err;
+
 			if ($stop_time) {
 				my $now = time;
 				if ( $stop_time < $now ) {
@@ -777,9 +808,6 @@ NAMESERVER: foreach my $ns (@ns) {
 					$timeout = $stop_time - $now;
 				}
 			}
-			my $nsname	 = $ns->[0];
-			my $nsaddr	 = $ns->[1];
-			my $nssockfamily = $ns->[2];
 
 			# If we do not have a socket for the transport
 			# we are supposed to reach the namserver on we
@@ -801,20 +829,22 @@ NAMESERVER: foreach my $ns (@ns) {
 				next NAMESERVER;
 			}
 
-			print ";; send_udp [$nsname]:$dstport\n"
-					if $self->{'debug'};
+			print ";; send_udp [$nsname]:$dstport\n" if $self->{'debug'};
 
 			unless ( $sock[$nssockfamily]->send( $packet_data, 0, $nsaddr ) ) {
-				print ";; send error: $!\n" if $self->{'debug'};
-				$self->errorstring("Send error: $!");
+				my $err = $ns->[3] = $self->errorstring("Send error: $!");
+				print ";; $err\n" if $self->{'debug'};
 				$nmbrnsfailed++;
-				$ns->[3] = "Send error" . $self->errorstring();
 				next;
 			}
 
-			# See ticket 11931 but this works not quite yet
-			my $oldpacket_timeout = time + $timeout;
-			until ( $oldpacket_timeout && ( $oldpacket_timeout < time() ) ) {
+			# handle failure to detect taint inside socket->send()
+			die 'Insecure dependency while running with -T switch' if tainted($nsaddr);
+
+
+			# See tickets #11931 and #97502
+			my $time_limit = time + $timeout;
+			while ( time() < $time_limit ) {
 				my @ready = $sel->can_read($timeout);
 		SELECTOR: foreach my $ready (@ready) {
 					my $buf = '';
@@ -885,8 +915,7 @@ NAMESERVER: foreach my $ns (@ns) {
 
 	if ( $sel->handles ) {
 
-		# If there are valid handles than we have either a timeout or
-		# a send error.
+		# If there are valid handles then we have either a timeout or a send error.
 		$self->errorstring('query timed out') unless ( $self->errorstring =~ /Send error:/ );
 	} else {
 		if ( $nmbrnsfailed < @ns ) {
@@ -916,7 +945,7 @@ sub bgsend {
 	my $srcaddr = $self->{'srcaddr'};
 	my $srcport = $self->{'srcport'};
 
-	my ( @res, $sockfamily, $dst_sockaddr );
+	my ( $sockfamily, $dst_sockaddr );
 	my ($ns_address) = @ns;
 	my $dstport = $self->{'port'};
 
@@ -926,27 +955,22 @@ sub bgsend {
 	# to the 'classic' mechanism
 	if ( $has_inet6 && !$self->force_v4() ) {
 
-		my ( $socktype_tmp, $proto_tmp, $canonname_tmp );
-
 		no strict 'subs';				# Because of the eval statement in the BEGIN
 								# AI_NUMERICHOST is not available at compile time.
 
-		my $old_wflag = $^W;				#circumvent perl -w warnings about 'udp'
-		$^W = 0;
+		local $^W = 0;					# circumvent perl -w warnings about 'udp'
 
-		# The AI_NUMERICHOST suppresses lookups.
 		my @res = Socket6::getaddrinfo( $ns_address, $dstport, AF_UNSPEC, SOCK_DGRAM, 0, AI_NUMERICHOST );
-
-		$^W = $old_wflag;
-
-		use strict 'subs';
-
-		( $sockfamily, $socktype_tmp, $proto_tmp, $dst_sockaddr, $canonname_tmp ) = @res;
-
 		if ( scalar(@res) < 5 ) {
-			die("can't resolve \"$ns_address\" to address (it could have been an IP address)");
+			for ($ns_address) {
+				die("invalid scoped address '$_'") if /[%].*$/;
+				die("invalid IP address '$_'");
+			}
 		}
 
+		my ( $socktype, $proto, $canonname );
+		( $sockfamily, $socktype, $proto, $dst_sockaddr, $canonname ) = @res;
+
 	} else {
 		$sockfamily = AF_INET;
 
@@ -994,6 +1018,9 @@ sub bgsend {
 			$self->errorstring("Send: [$ns_address]:$dstport  $!");
 			print ";; ", $self->errorstring(), "\n" if $self->{'debug'};
 		}
+
+		# handle failure to detect taint inside socket->send()
+		die 'Insecure dependency while running with -T switch' if tainted($dst_sockaddr);
 		return $socket;
 	}
 	$self->errorstring("Could not find a socket to send on");
@@ -1049,22 +1076,17 @@ sub make_query_packet {
 
 	$header->rd( $self->{recurse} ) if $header->opcode eq 'QUERY';
 
-	if ( $self->dnssec ) {					# RFC 3225
-		print ";; Set EDNS DO flag and UDP packetsize $self->{udppacketsize}\n" if $self->{debug};
-		$packet->edns->size( $self->{udppacketsize} );	# advertise UDP payload size for local IP stack
+	$header->ad(1) if $self->{adflag};			# RFC6840, 5.7
+	$header->cd(1) if $self->{cdflag};			# RFC6840, 5.9
+
+	if ( $self->dnssec ) {
+		print ";; Set EDNS DO flag\n" if $self->{debug};
+		$header->ad(0);
 		$header->do(1);
-		$header->ad( $self->{adflag} );
-		$header->cd( $self->{cdflag} );
-
-	} elsif ( $self->{udppacketsize} > PACKETSZ ) {
-		print ";; Clear EDNS DO flag and set UDP packetsize $self->{udppacketsize}\n" if $self->{debug};
-		$packet->edns->size( $self->{udppacketsize} );	# advertise UDP payload size for local IP stack
-		$header->do(0);
-
-	} else {
-		$header->do(0);
 	}
 
+	$packet->edns->size( $self->{udppacketsize} );		# advertise payload size for local stack
+
 	if ( $self->{tsig_rr} && !grep $_->type eq 'TSIG', $packet->additional ) {
 		$packet->sign_tsig( $self->{tsig_rr} );
 	}
@@ -1076,31 +1098,68 @@ sub make_query_packet {
 sub axfr {				## zone transfer
 	my $self = shift;
 
+	my $whole = wantarray;
 	my @null;
-	my $query = $self->_axfr_start(@_) || return @null;
+	my $query = $self->_axfr_start(@_) || return $whole ? @null : sub {undef};
+	my $reply = $self->_axfr_next()	   || return $whole ? @null : sub {undef};
+	my @rr	  = $reply->answer;
+	my $soa	  = $rr[0];
+	my $verfy = $query->sigrr();
+	$verfy = $reply->verify($query) || croak $reply->verifyerr if $verfy;
+	print ';; ', $verfy ? '' : 'not ', "verified\n" if $self->{debug};
 
-	my $reply = $self->_axfr_next() || return @null;	# initial packet
-	my $vrify = $reply->verify($query) || croak $reply->verifyerr if $query->sigrr;
+	if ($whole) {
+		my @zone = shift @rr;
 
-	my ( $soa, @rr ) = $reply->answer;
-	my @zone = ($soa);
+		until ( scalar @rr && $rr[$#rr]->type eq 'SOA' ) {
+			push @zone, @rr;			# unpack non-terminal packet
+			@rr    = @null;
+			$reply = $self->_axfr_next() || last;
+			$verfy = $reply->verify($verfy) || croak $reply->verifyerr if $verfy;
+			print ';; ', $verfy ? '' : 'not ', "verified\n" if $self->{debug};
+			@rr = $reply->answer;
+		}
 
-	until ( scalar grep $_->type eq 'SOA', @rr ) {		# unpack non-terminal packet(s)
+		my $last = pop @rr;				# unpack final packet
 		push @zone, @rr;
-		@rr    = @null;
-		$reply = $self->_axfr_next() || last;
-		$vrify = $reply->verify($vrify) || croak $reply->verifyerr if $query->sigrr;
-		@rr    = $reply->answer;
+		$self->{axfr_sel} = undef;
+		croak 'improperly terminated AXFR' unless $last && $last->encode eq $soa->encode;
+		return @zone;
 	}
 
-	$self->{axfr_sel} = undef;
+	return sub {			## iterator over RRs
+		my $rr = shift @rr;
+		croak 'improperly terminated AXFR' unless $rr;
+		return $rr if scalar @rr;
 
-	foreach my $rr (@rr) {					# unpack final packet
-		return @zone if $rr->type eq 'SOA' && $rr->string eq $soa->string;
-		push @zone, $rr;
-	}
+		if ( $rr->type eq 'SOA' ) {
+			unless ( $rr eq $soa ) {		# start of zone
+				$self->{axfr_sel} = undef;	# end of zone
+				croak 'improperly terminated AXFR' unless $rr->encode eq $soa->encode;
+				return undef;
+			}
+		}
 
-	croak 'improperly terminated AXFR';
+		$reply = $self->_axfr_next() || return undef;	# end of packet
+		$verfy = $reply->verify($verfy) || croak $reply->verifyerr if $verfy;
+		print ';; ', $verfy ? '' : 'not ', "verified\n" if $self->{debug};
+		@rr = $reply->answer;
+		return $rr;
+	};
+}
+
+
+sub axfr_start {			## historical
+	my $self = shift;
+	my $iter = $self->{axfr_iter} = $self->axfr(@_);
+	return defined($iter) || undef;
+}
+
+
+sub axfr_next {				## historical
+	my $self = shift;
+	my $iter = $self->{axfr_iter} || return undef;
+	$iter->() || return $self->{axfr_iter} = undef;
 }
 
 
@@ -1109,8 +1168,7 @@ sub _axfr_start {
 	my $dname = shift || $self->{'searchlist'}->[0];
 	my $class = shift || 'IN';
 
-	my $debug   = $self->{debug};
-	my $timeout = $self->{tcp_timeout};
+	my $debug = $self->{debug};
 
 	unless ($dname) {
 		$self->errorstring('no zone specified');
@@ -1167,20 +1225,19 @@ sub _axfr_next {
 	my $self = shift;
 
 	my $debug = $self->{debug};
-	unless ( $self->{axfr_sel} ) {
+	my $sel	  = $self->{axfr_sel};
+	unless ($sel) {
 		$self->errorstring('no zone transfer in progress');
 		print ';; ', $self->errorstring, "\n" if $debug;
 		return;
 	}
 
-	my $sel	    = $self->{axfr_sel};
-	my $timeout = $self->{tcp_timeout};
-
 	#--------------------------------------------------------------
 	# Read the length of the response packet.
 	#--------------------------------------------------------------
 
-	my @ready = $sel->can_read($timeout);
+	my $timeout = $self->{tcp_timeout};
+	my @ready   = $sel->can_read($timeout);
 	unless (@ready) {
 		$self->errorstring('timeout');
 		return;
@@ -1261,14 +1318,9 @@ sub tsig {
 sub dnssec {
 	my $self = shift;
 
-	unless (DNSSEC) {
-		carp 'resolver->dnssec(1) without Net::DNS::SEC installed' if shift;
-		return $self->{dnssec} = 0;
-	}
-
 	return $self->{dnssec} unless scalar @_;
 
-	# set flag and increase default udppacket size
+	# increase default udppacket size if flag set
 	$self->udppacketsize(2048) if $self->{dnssec} = shift;
 
 	return $self->{dnssec};
@@ -1399,44 +1451,60 @@ sub _create_tcp_socket {
 }
 
 
-# Lightweight versions of subroutines from Net::IP module, recoded to fix rt#28198
+# Lightweight versions of subroutines from Net::IP module, recoded to fix RT#96812
 
 sub _ip_is_ipv4 {
-	my @field = split /\./, shift;
-
-	return 0 if @field > 4;					# too many fields
-	return 0 if @field == 0;				# no fields at all
-
-	foreach (@field) {
-		return 0 unless /./;				# reject if empty
-		return 0 if /[^0-9]/;				# reject non-digit
-		return 0 if $_ > 255;				# reject bad value
-	}
-
-	return 1;
+	return shift =~ /^[0-9.]+\.[0-9]+$/;			# dotted digits
 }
 
 
 sub _ip_is_ipv6 {
 
 	for (shift) {
-		my @field = split /:/;				# split into fields
-		return 0 if ( @field < 3 ) or ( @field > 8 );
-
-		return 0 if /::.*::/;				# reject multiple ::
-
-		if (/\./) {					# IPv6:IPv4
-			return 0 unless _ip_is_ipv4( pop @field );
-		}
-
-		foreach (@field) {
-			next unless /./;			# skip ::
-			return 0 if /[^0-9a-f]/i;		# reject non-hexdigit
-			return 0 if length $_ > 4;		# reject bad value
-		}
+		return 1 if /^[:0-9a-f]+:[0-9a-f]*$/i;		# mixed : and hexdigits
+		return 1 if /^[:0-9a-f]+:[0-9a-f]*[%].+$/i;	# RFC4007 scoped address
+		return 1 if /^[:0-9a-f]+:[0-9.]+$/i;		# prefix + dotted digits
 	}
 
-	return 1;
+	return 0;
+}
+
+
+sub force_v4 {
+	my $self = shift;
+	return $self->{force_v4} unless scalar @_;
+	my $value = shift;
+	$self->force_v6(0) if $value;
+	$self->{force_v4} = $value ? 1 : 0;
+}
+
+sub force_v6 {
+	my $self = shift;
+	return $self->{force_v6} unless scalar @_;
+	my $value = shift;
+	$self->force_v4(0) if $value;
+	$self->{force_v6} = $value ? 1 : 0;
+}
+
+sub prefer_v4 {
+	my $self = shift;
+	return $self->{prefer_v6} ? 0 : 1 unless scalar @_;
+	my $value = shift;
+	$self->{prefer_v6} = $value ? 0 : 1;
+	return $value;
+}
+
+sub prefer_v6 {
+	my $self = shift;
+	return $self->{prefer_v6} unless scalar @_;
+	$self->{prefer_v6} = shift() ? 1 : 0;
+}
+
+
+sub udppacketsize {
+	my $self = shift;
+	$self->{udppacketsize} = shift if scalar @_;
+	return $self->_packetsz;
 }
 
 
@@ -1450,7 +1518,7 @@ sub AUTOLOAD {				## Default method
 
 	my $name = $AUTOLOAD;
 	$name =~ s/.*://;
-	croak "$name: no such method" unless exists $self->{$name};
+	croak "$name: no such method" unless exists $public_attr{$name};
 
 	no strict q/refs/;
 	*{$AUTOLOAD} = sub {
@@ -1470,7 +1538,7 @@ __END__
 
 =head1 NAME
 
-Net::DNS::Resolver::Base - Common Resolver Class
+Net::DNS::Resolver::Base - DNS resolver base class
 
 =head1 SYNOPSIS
 
@@ -1484,18 +1552,36 @@ sub-classes of L<Net::DNS::Resolver>.
 No user serviceable parts inside, see L<Net::DNS::Resolver>
 for all your resolving needs.
 
+
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr.
+Copyright (c)2003,2004 Chris Reinhardt.
 
-Portions Copyright (c) 2002-2004 Chris Reinhardt.
+Portions Copyright (c)2005 Olaf Kolkman.
 
-Portions Copyright (c) 2005 Olaf Kolkman.
+Portions Copyright (c)2006,2014 Dick Franks.
 
-Portions Copyright (c) 2006,2014 Dick Franks.
+All rights reserved.
+
+
+=head1 LICENSE
+
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted, provided
+that the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation, and that the name of the author not be used in advertising
+or publicity pertaining to distribution of the software without specific
+prior written permission.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
